// ====================================================================
// GLCanvas class by Rob Jagnow.
//
// The GLCanvas can be created with a call to the 'initialize' routine,
// which takes as parameters a pointer to the global SceneParser and
// the name of the routine that will perform the ray tracing.  Once the
// OpenGL interface is open, the scene can be rendered from the current
// camera position by pressing the 'r' key.
// ====================================================================

#ifndef _GL_CANVAS_H_
#define _GL_CANVAS_H_

#include <stdlib.h>

#include "argparser.h"
#include "camera.h"
#include "grid.h"

// ====================================================================

class GLCanvas {

private:

  // State of the mouse cursor
  static int mouseButton;
  static int mouseX;
  static int mouseY;

  // Callback functions for mouse and keyboard events
  static void display(void);
  static void reshape(int w, int h);
  static void mouse(int button, int state, int x, int y);
  static void motion(int x, int y);
  static void keyboard(unsigned char key, int x, int y);
  static void idle();

  static ArgParser *args;
  static Camera *camera;
  static Grid *grid;

  static void InitLight();

  static int display_list_index;

public:
  // Constructor and destructor
  GLCanvas(void) { }
 ~GLCanvas(void) { }

  // Set up the canvas and enter the rendering loop
  // Note that this function will not return but can be
  // terminated by calling 'exit(0)'

 void initialize(ArgParser *_args, Grid *_grid);
 static void Render();

};

int HandleGLError();

// ====================================================================

#endif
